#!/usr/bin/bash
#                                                                                                                                                                                   #
#    Author: Maria Beatriz Walter Costa (bia@bioinf.uni-leipzig.de)                                                                                                                 #
#                                                                                                                                                                                   #
#    This script processes a multi-FASTA file of non-coding RNAs, either previously aligned or not, and returns for each species selection scores in relation to their structure    #
# and considering the input group.                                                                                                                                                  #
#                                                                                                                                                                                   #
#    Input multi-FASTA file has to be in the appropriate format and in a separate folder. The name of the file must correspond to the ID of the group followed by e file extension. #
# - Examples: identity.fasta, identity.fa, identity.alg                                                                                                                             #
#                                                                                                                                                                                   #
# The header of the fasta sequences must have the header indicator '>' followed immediatly by the ID of the group (the same name of the file without extension), followed by a tab, #
# followed by a pipe sign, followed by a second tab, followed by the species name or sequence code. Minus sings are prohibitted, but underscores are allowed.                       # 
# - Examples: >identity	|	Species                                                                                                                                             #
#                                                                                                                                                                                   #
#    Usage: bash SSS.sh folder/identity.fa FORMAT (fasta/aligned)  Structure(Yes/No)                                                                                                #
#                                                                                                                                                                                   #
# Output header:                                                                                                                                                                    #
# species_ID	number_ch	sp_dist	sp_length	alg_length	fisher_indels	fisher_pvalue1	sp_score sp_dist_median                                                     #
#                                                                                                                                                                                   #
# - number_ch refers to the number of species specific substitutions (calculated in every row of the alignment). A species specific substitution is a base of the species that      #
# differs from the dominant base of the row, when a dominant row exists. A dominant base exists when at least 60% of the bases of the row converge;                                 #
# - sp_dist is the structural distance between the species and the consensus;                                                                                                       #
# - fisher_indels is the indels score, calculated using the Fisher's method;                                                                                                        #
# - fisher_pvalue is the substitutions score, calculated using the Fisher's method;                                                                                                 #
# - sp_score is the species selection score;                                                                                                                                        #
# - divergence is the family structural divergence.                                                                                                                                 #
#                                                                                                                                                                                   #
#####################################################################################################################################################################################

if [ $# -lt 3 ]; then
	echo "Usage: bash $0 InputFolder/File Format Structure"
  	echo "Format: fasta or aligned"
	echo "Structure: Yes/No"
  	exit 1
fi

if [[ $1 =~ "-" ]]; then
	echo "Input file must be in specific format, and can not contain '-'";
	echo "Example allowed names: 'input.fa', 'input.alg', 'input_100.fa', 'input_100.alg'";
	exit
fi

if [[ ! -s $1 ]]; then
	echo "It was not possible to open file $1"
	exit
fi

FORMAT="$2"
STRUCTURE="$3"

f1=$1
f2=${f1%/*}
f3=${f1##*/}
f4=${f3%.*}

if [[ -d "$f4" ]]
   then
	rm -r $f4
fi

mkdir $f4
cd $f4/

perl ../scripts/get-species.pl ../$f1 > species-$f4.txt;

NUM_OF_SEQ=$(wc -l < species-$f4.txt); 

if (( $NUM_OF_SEQ < 3 ))
   then

	echo "$f4 has only $NUM_OF_SEQ species, which is not enough information to calculate the sd-ss scores";
	echo "You should have at least 3 species";
	
	cd ../
	rm -r $f4

	exit
fi

#export RNASNPPATH=/usr/local/RNAsnp-1.1;

if [ $FORMAT == "fasta" ]; then 
	echo 'Aligning sequences'
	muscle -in ../$f1 > $f4.alg;
	echo ''

	perl ../scripts/twoline-fasta.pl ../$f1 > $f4-twoline; 
	perl ../scripts/twoline-fasta.pl $f4.alg > $f4.alg-twoline; 

	echo 'Calculating alignment changes'
	perl ../scripts/alignment_changes-nogap2.pl $f4.alg-twoline 0.6 > $f4.changes; 

	echo 'Creating individual FASTA files'
	perl ../scripts/IO.pl $f4-twoline fa;

	mv $f4.alg-twoline alignment.alg;
	perl ../scripts/IO.pl alignment.alg alg;
	rm $f4-twoline $f4.alg;
fi

if [ $FORMAT == "aligned" ]; then
	echo 'Getting raw multi-FASTA'
	perl ../scripts/twoline-fasta.pl ../$f1 > $f4-twoline; 
	perl ../scripts/nogap.pl $f4-twoline > $f4.fa;

	echo 'Calculating alignment changes'
	perl ../scripts/alignment_changes-nogap2.pl $f4-twoline 0.6 > $f4.changes; 

	echo 'Creating individual FASTA files'
	perl ../scripts/IO.pl $f4.fa fa; 
	rm $f4.fa;

	mv $f4-twoline alignment.alg; 
	perl ../scripts/IO.pl alignment.alg alg;
fi

if [ -a ../$f2/$f4.sss ]; then
	rm ../$f2/$f4.sss
fi

echo "species_ID	number_ch	dist_sum	sp_dist	sp_length	alg_length	indel_score_list	fisher_indels	fisher_pvalue1	fisher_pvalue2	sp_score" >> $f4-nomedian.sss;
while read species
  do
	echo "Computing $species..."

	#Creating a new alignment, excluding the current species 
	perl ../scripts/convert-format.pl alignment.alg > align.alg
	perl ../scripts/alignment-without-species.pl align.alg $species > new-alg.alg;

	echo "Getting consensus sequence from RNAalifold (without $species)"
	perl ../scripts/convertAlignment.pl -i new-alg.alg -o $f4-$species.clw -f clustalw; 
	rm new-alg.alg

	RNAalifold $f4-$species.clw > $f4-$species.alifold;

	echo ">$f4-$species-consensus" >> $f4-$species-consensus.alg;
	head -n1 $f4-$species.alifold >> $f4-$species-consensus.alg; 

	head -n1 $f4-$species.alifold > cons.txt	
	cons=`wc -c < cons.txt`
	cons=$((cons-1)) 
	rm cons.txt

	if [[ -f ../$f2/$f4-$species-consensus.rnasnp ]]
	   then 
		cp ../$f2/$f4-$species-consensus.rnasnp .
	   else
		echo "Measuring change probability in consensus sequence using RNAsnp"
		RNAsnp --pvalue1=10.0 --pvalue2=10.0 -f $f4-$species-consensus.alg -m 3 > $f4-$species-consensus.rnasnp; 
		cp $f4-$species-consensus.rnasnp ../$f2/.
	fi

	perl ../scripts/analyse-RNAsnp.pl $f4-$species-consensus.rnasnp > $f4-$species-consensus.tmp0;
	perl ../scripts/tmp0_aligned.pl $f4-$species-consensus.tmp0 $f4-$species-consensus.alg > $f4-$species-consensus.tmp1;

	echo 'Folding sequences with RNAfold'
	RNAfold -p -d2 --noLP < $f4-$species-consensus.alg  > $f4-$species-consensus.out; 

	perl ../scripts/intersect-changes-rnasnp.pl $f4.changes $f4-$species-consensus.tmp1 $species > $f4-$species.tmp2;

	if grep 'The species you want to search' $f4-$species.tmp2
	   then
		echo 'Error in input format file'
		cd ..
		rm $f4/*
		rmdir $f4/
		exit
	fi

	RNAfold -p -d2 --noLP < $f4-$species.alg > $f4-$species.out;

	mv ${f4}_dp.ps $f4-${species}_dp.ps
	mv ${f4}_ss.ps $f4-${species}_ss.ps

	relplot.pl -p $f4-$species-consensus_ss.ps $f4-$species-consensus_dp.ps > $f4-$species-consensus_rss.ps
	relplot.pl -p $f4-${species}_ss.ps $f4-${species}_dp.ps > $f4-${species}_rss.ps

	echo "Computing compensatory pairs between minimum free-energy structures of $species and consensus"

	#Computing compensatory pairs, based on the alignment positions of species and consensus
	perl ../scripts/compensatory.pl $f4-$species.tmp2 $f4-$species-consensus_dp.ps $f4-${species}_dp.ps > $f4-$species.tmp2.1; 

	alg_length=`grep -v '>' alignment.alg | head -1`;
	perl ../scripts/ensemble-distance.pl $f4-$species-consensus_dp.ps $f4-${species}_dp.ps ${#alg_length} > $f4-$species.distance;
	distance=`cat $f4-$species.distance`;
		
	#Computing indels and creating mutated-sequence files in FASTA format
	#Mutated files are species-mutated sequences, each one being the species sequence plus one indel
	echo 'Creating indel-mutated species FASTA files'
	perl ../scripts/indel-extended.pl $f4-$species.alifold $f4-$species.alg > $species-indel.tmp;

	#Getting `ancestral_structure' for the observed indels impact analysis using the group consensus
	#The consensus is considered the ancestral because it represents the group, then we can calculate the impact of the observed indels calculating how they diverge from the group
	perl ../scripts/get-structure.pl $f4-$species-consensus.out > ancestral_structure.txt;	

	ch=`head -n1 $species-indel.tmp`

	if [ "$ch" -gt "0" ];
	   then
		echo 'Calculating structural distances between indel-mutated and consensus using RNAforester'

		#Constructing species observed indels
		for i in mutated*.fa
		   do
			RNAfold -p -d2 --noLP < $i > noconstraint.out; 	
			perl ../scripts/get-structure.pl noconstraint.out > noconstraint.txt;
			echo ">noconstraint" > noconstraint.vienna
			grep -v '>' $i >> noconstraint.vienna
			cat noconstraint.txt >> noconstraint.vienna

			cat $i ancestral_structure.txt > $i.constraint
			RNAfold -p -d2 --noLP -C < $i.constraint > constraint.out; 	
			perl ../scripts/get-structure.pl constraint.out > constraint.txt;
			echo ">constraint" > constraint.vienna
			grep -v '>' $i >> constraint.vienna
			cat constraint.txt >> constraint.vienna

			cat constraint.vienna noconstraint.vienna > comparator.txt

			#Command below substitute eventual 'N's in the sequences of indel-mutated and/or consensus for random base 'a' - This is necessary for RNAforester to run! If the input contain N's, RNAforester does not recognise the input format and aborts
			sed 's/N/a/g' comparator.txt > comparator.txt2

			RNAforester -d < comparator.txt2 > tmp.out
			perl ../scripts/indel-impact.pl tmp.out $i >> $f4-$species-observed.indel
			rm $i noconstraint.out noconstraint.txt noconstraint.vienna constraint.out constraint.txt constraint.vienna comparator.txt2 comparator.txt tmp.out $i.constraint ${i%.*}_ss.ps ${i%.*}_dp.ps	
		done

	#Rank statistics calculation for species indels
	echo 'Rank statistics for getting indel probabilities'

	for GAP_LENGTH in `tail -n1 $species-indel.tmp`
	   do
		if [[ -f ../$f2/$f4-$species-$GAP_LENGTH.indel ]]
		   then
			cp ../$f2/$f4-$species-$GAP_LENGTH.indel .
		   else
			echo "Calculating background information of structural impact of a $GAP_LENGTH bp gap on the consensus using the gap-modelling pipeline and RNAforester"
			grep '>' $f4-$species-consensus.alg > $f4-$species-consensus.alg2
			grep -v '>' $f4-$species-consensus.alg | sed 's/_/-/g' >> $f4-$species-consensus.alg2
			sh ../scripts/gap-modelling.sh $f4-$species-consensus.alg2 $f4-$species-consensus.out $GAP_LENGTH $f4-$species
			cp $f4-$species-$GAP_LENGTH.indel ../$f2/. 
			rm $f4-$species-consensus.alg2
		fi
		cat $f4-$species-$GAP_LENGTH.indel | cut -f4 | sort -n | grep -v RNA | uniq -c > tmp.indel
		sed 's/  */ /g' tmp.indel > $f4-$species-$GAP_LENGTH.indelgroup #Required by script indel-score.pl 
		rm tmp.indel $f4-$species-$GAP_LENGTH.indel
	done
	else
		echo 'no_indels' >> $f4-$species-observed.indel
	fi
	rm -f ancestral_structure.txt

	echo "Correcting species specific changes p-values and indel scores for multiple observation using the Bonferroni method"
	echo "Calculating scores for species specific changes and indels using Fisher's method for combining p-values"
	perl ../scripts/indel-score.pl $f4-$species-observed.indel $f4-$species > $f4-$species.indel

	perl ../scripts/pvalues-bonferroni.pl $f4-$species.tmp2.1 > $f4-$species.pvalue;
	perl ../scripts/indels-bonferroni.pl $f4-$species.indel > indel.final;
 
	echo "Combining both scores for species specific changes and indels into one final selection score"
	perl ../scripts/sum-all.pl $f4-$species.tmp2.1 $distance $f4 $species ${#alg_length} $f4-$species.indel $f4-$species.pvalue indel.final >> $f4-nomedian.sss;
	cp $f4-$species.pvalue ../$f2/.

	rm $f4-$species-consensus.tmp0 $f4-$species.distance $f4-$species.tmp2 $f4-$species.tmp2.1 $f4-$species.pvalue $f4-$species.alg $f4-$species-consensus.tmp1 $f4-$species.clw $f4-$species.alifold alirna.ps $f4-$species-consensus.rnasnp $species-indel.tmp $f4-${species}_ss.ps $f4-$species-consensus_ss.ps $f4-$species.out $f4-$species-consensus.out indel.final $f4-$species-observed.indel $f4-$species.indel $f4-${species}_dp.ps $f4-${species}_rss.ps $f4-${species}-consensus_dp.ps $f4-${species}-consensus_rss.ps
	if [ "$ch" -gt "0" ];
	   then
		rm  $f4-$species-*.indelgroup
	fi
done < species-$f4.txt

#Getting the distance median of the family
median=`perl ../scripts/median.pl $f4-nomedian.sss`
perl ../scripts/add-median.pl $f4-nomedian.sss $median > $f4-full.sss
perl ../scripts/filter-output.pl $f4-full.sss > ../$f2/$f4.sss;

cat ../$f2/$f4.sss;
rm $f4.changes alignment.alg align.alg $f4-nomedian.sss $f4-full.sss

if [[ $STRUCTURE == "Yes" ]] 
   then

	if [[ -d "../$f2/${f4}_structures" ]]
	   then
		rm -r ../$f2/${f4}_structures
	fi

	mkdir ../$f2/${f4}_structures

	while read species
	do
		#Obtaining secondary structures of the consensus and species (no gaps included)
		head -n1 $f4-$species-consensus.alg >> $f4-$species-consensus.fa
		grep -v '>' $f4-$species-consensus.alg | sed 's/[-_]//g' >> $f4-$species-consensus.fa

		RNAfold -p -d2 --noLP < $f4-$species-consensus.fa > $f4-$species-consensus.out;

		relplot.pl -p $f4-${species}-consensus_ss.ps $f4-${species}-consensus_dp.ps > $f4-${species}-consensus_rss.ps

		RNAfold -p -d2 --noLP < $f4-$species.fa > $f4-$species.out;
		mv ${f4}_dp.ps $f4-${species}_dp.ps
		mv ${f4}_ss.ps $f4-${species}_ss.ps
		relplot.pl -p $f4-${species}_ss.ps $f4-${species}_dp.ps > $f4-${species}_rss.ps

		mv $f4-${species}_rss.ps $f4-${species}-consensus_rss.ps $f4-${species}-consensus_dp.ps $f4-${species}_dp.ps ../$f2/${f4}_structures/

		rm $f4-$species-consensus.out $f4-$species.out $f4-$species-consensus.alg $f4-$species-consensus.fa $f4-$species.fa $f4-${species}_ss.ps $f4-${species}-consensus_ss.ps
	done < species-$f4.txt

   elif [[ $STRUCTURE == "No" ]]
      then
	if [[ -d "../$f2/${f4}_structures" ]]
	   then
		rm -r ../$f2/${f4}_structures
	fi
	rm $f4-*-consensus.alg $f4-*.fa
fi

rm species-$f4.txt 
cd ../
rmdir $f4


