#!/usr/bin/bash
#                                                                                                                                                                                   #
#    Author: Maria Beatriz Walter Costa (bia@bioinf.uni-leipzig.de)                                                                                                                 #
# This script processes a multi-FASTA file of long non-coding RNAs, aligned or not, and returns multi-FASTA not aligned local structure separate files. These multi FASTA files can #
# afterwards be submitted to the SSS-test as a non-aligned lncRNA family (fasta mode). The local structures are found based on RNALfold reports and classified as orthologous local #
# structures based on the lncRNA alignment positions. They must overlap at least 70%.                                                                                               #
#                                                                                                                                                                                   #
#multi-FASTA has to be in appropriate format and in a separate folder, in the same formatting as in the SSS-test.                                                                   #
#                                                                                                                                                                                   #
# Usage: bash local.sh folder/lncRNA.fa Format (fasta or aligned)                                                                                                                   #
#                                                                                                                                                                                   #
#####################################################################################################################################################################################

if [ $# -lt 2 ]; then
	echo "Usage: $0 InputFolder/File Format"
  	echo "Format: fasta or aligned"
  	exit 1
fi

if [[ ! -s $1 ]]; then
	echo "It was not possible to open file $1"
	exit
fi

if [[ $1 =~ "-" ]]; then
	echo "Input file must be in specific format, and can not contain '-'"
	echo "Example allowed names: 'input.fa', 'input.alg', 'input_100.fa', 'input_100.alg'"
	exit
fi

FORMAT="$2"
input="$1"

folder=${input%/*} 
filename=${input##*/} 
file=${filename%.*} 

if [ -d "${file}_local" ]; then
	echo "${file}_local already exists"
	echo "Remove old directory of local structures before running local structure pipeline"
	exit
fi

mkdir ${file}_local
cd ${file}_local

if [ $FORMAT == "fasta" ]; then 
	echo 'Aligning sequences'
	muscle -in ../$input > $file.alg;
	echo ''

	perl ../scripts/twoline-fasta.pl ../$input > $file-twoline; 
	perl ../scripts/twoline-fasta.pl $file.alg > $file.alg-twoline; 

	rm $file.alg
	mv $file.alg-twoline $file.alg
	mv $file-twoline $file.fa
fi

if [ $FORMAT == "aligned" ]; then 
	echo 'Getting raw multi-FASTA'
	perl ../scripts/twoline-fasta.pl ../$input > $file.alg-twoline; 
	mv $file.alg-twoline $file.alg

	perl ../scripts/nogap.pl $file.alg > $file.fa;
fi

echo 'Creating individual FASTA files'
perl ../scripts/IO.pl $file.fa fa;
perl ../scripts/get-species.pl $file.alg > species.tmp;

NUM_OF_SEQ=$(wc -l < species.tmp); 

if [[ $NUM_OF_SEQ < 2 ]]
   then

	echo "$f4 has only $NUM_OF_SEQ species, which is not enough information to assign orthologous families local structures";
	echo "You should have at least 2 species to assign orthologous local structures, and at least 3 to calculate sd-ss scores";
	
	cd ../
	rm -r $f4

	exit
fi

#Local structure search with RNALfold
echo "Looking for local structures with RNALfold..."

while read species
   do
	RNALfold -z 2 < $file-$species.fa > $file-$species.localfold;

	#Correction of RNALfold format
	awk '{if(length($2)<=2){print $1 "\t("$3"\t"$4"\t"$5"\t"$6} else {print $0}}' $file-$species.localfold > $file-$species.localfold.formated;
	sort -k3 -n $file-$species.localfold.formated > $file-$species.localfold.formated.sorted;
	perl ../scripts/best-substructures.pl $file-$species.localfold.formated.sorted $species > $file-$species.best;
	sed 's/  */\t/g' $file-$species.best > $file-$species.structures

	cat $file-$species.structures >> $file.local

	
	rm $file-$species.localfold $file-$species.localfold.formated $file-$species.localfold.formated.sorted $file-$species.best $file-$species.structures $file-$species.fa
done < species.tmp

rm species.tmp

echo "Extracting local structure sequences"
perl ../scripts/extract-subsequences.pl $file.local $file.alg > substructures.fa;

echo "Assigning orthologous local sequence families"
perl ../scripts/assign-orthologous.pl substructures.fa > substructures-ortho.fa

echo "Creating table of local structure alignment positions"
echo $file > ../$file-local_positions.txt
perl ../scripts/sub-table.pl substructures-ortho.fa > ../$file-local_positions.txt  

echo "Creating local sequence family files"
perl ../scripts/local-files.pl substructures-ortho.fa

rm substructures.fa substructures-ortho.fa $file.alg $file.local $file.fa

cd ../



